@@ -20,25 +20,19 @@
 ccopy <- function(from,
                   to,
                   recursive = TRUE) {
-  if (is.null(from) || !is.character(from)) {
-    stop("from must be a character", call. = FALSE)
-  }
-  if (is.null(to) || !is.character(to)) {
-    stop("to must be a character", call. = FALSE)
-  }
-  if (is.null(from) || !is.logical(recursive)) {
-    stop("recursive must be a logical", call. = FALSE)
-  }
+  check_arg(from, "character", n = 1)
+  check_arg(to, "character", n = 1)
+  check_arg(recursive, "logical", n = 1)
 
   from_list <- dir(path = from, pattern = "*.zip$", recursive = recursive)
   to_list <- dir(path = to, pattern = "*.zip$", recursive = recursive)
   copy <- setdiff(from_list, to_list)
 
   if (length(copy) == 0) {
-    return(message("No files left to copy"))
+    return(inform("No files left to copy"))
   }
 
-  message(paste0("Copying ", length(copy), " files."))
+  inform(paste0("Copying ", length(copy), " files."))
   to_copy <- file.path(from, copy)
   invisible(do.call(
     file.copy,
@@ -49,12 +43,33 @@ ccopy <- function(from,
 #' Fix the end of JSON files
 #'
 #' @description
-#' `r lifecycle::badge("stable")`
+#' `r lifecycle::badge("experimental")`
 #'
 #' When copying data directly coming from m-Path Sense, JSON files are sometimes corrupted due to
 #' the app not properly closing them. This function attempts to fix the most common
 #' problems associated with improper file closure by m-Path Sense.
 #'
+#' @details
+#' There are two distinct problems this functions tries to tackle. First of all, there are often
+#' bad file endings (e.g. no \code{]}) because the app was closed before it could properly close
+#' the file. There are several cases that may be wrong (or even multiple), so it unclear what the
+#' precise problems are. As this function is experimental, it may even make it worse by accidentally
+#' inserting an incorrect file ending.
+#'
+#' Secondly, in rare scenarios there are illegal ASCII characters in the JSON files. Not often does
+#' this happen, and it is likely because of an OS failure (such as a flush error), a disk failure,
+#' or corrupted data during transmit. Nevertheless, these illegal characters make the file
+#' completely unreadable. Fortunately, they are detected correctly by
+#' \link[mpathsenser]{test_jsons}, but they cannot be imported by \link[mpathsenser]{import}. This
+#' functions attempts to surgically remove lines with illegal characters, by removing that specific
+#' line as well as the next line, as this is often a comma. It may therefore be too liberal in its
+#' approach -- cutting away more data than necessary -- or not liberal enough when the corruption
+#' has spread throughout multiple lines. Nevertheless, it is a first step in removing some
+#' straightforward corruption from files so that only a small number may still need to be fixed by
+#' hand.
+#'
+#' @inheritSection import Parallel
+#'
 #' @inheritSection import Progress
 #'
 #' @param path The path name of the JSON files.
@@ -62,90 +77,79 @@ ccopy <- function(from,
 #' @param recursive Should the listing recurse into directories?
 #' @param parallel A logical value whether you want to check in parallel. Useful for a lot of files.
 #'
+#'   `r lifecycle::badge("deprecated")` As functions should not modify the user's workspace,
+#'   directly toggling parallel support has been deprecated. Please use
+#'   \code{\link[future]{plan}("multisession")} before calling this function to use multiple
+#'   workers.
+#'
 #' @return A message indicating how many files were fixed.
 #' @export
 #' @examples
 #' \dontrun{
-#' future::plan(future::multisession)
+#' future::plan("multisession")
 #' files <- test_jsons()
 #' fix_jsons(files = files)
 #' }
 fix_jsons <- function(path = getwd(),
                       files = NULL,
                       recursive = TRUE,
-                      parallel = FALSE) {
-  if (!requireNamespace("vroom", quietly = TRUE)) {
-    stop(paste0(
-      "package vroom is needed for this function to work. ",
-      "Please install it using install.packages(\"vroom\")"
-    ),
-    call. = FALSE
-    )
-  }
+                      parallel = deprecated()) {
+  ensure_suggested_package("vroom")
 
-  if (!is.null(path) && !is.character(path)) {
-    stop("path must be a character string of the path name")
-  }
-  if (!is.null(files) && !is.character(files)) {
-    stop("files must be NULL or a character vector of file names")
+  check_arg(path, "character", n = 1, allow_null = TRUE)
+  check_arg(files, "character", allow_null = TRUE)
+  check_arg(recursive, "logical", n = 1)
+
+  if (is.null(path) && is.null(files)) {
+    abort("`path` and `files` cannot be NULL at the same time.")
   }
 
   # Find all JSON files that are _not_ zipped Thus, make sure you didn't unzip them yet,
   # otherwise this may take a long time
-  if (is.null(files)) {
-    jsonfiles <- dir(
+  if (!is.null(path) && is.null(files)) {
+    jsonfiles <- list.files(
       path = path,
       pattern = "*.json$",
       all.files = TRUE,
       recursive = recursive,
       full.names = TRUE
     )
+  } else if (!is.null(path) && !is.null(files)) {
+    jsonfiles <- normalizePath(file.path(path, files), mustWork = TRUE)
   } else {
-    if (!missing(path)) {
-      tryCatch(
-        {
-          normalizePath(file.path(path, files), mustWork = TRUE)
-        },
-        error = function(e) stop(e)
-      )
-      jsonfiles <- normalizePath(file.path(files))
-    } else {
-      tryCatch(
-        {
-          normalizePath(files, mustWork = TRUE)
-        },
-        error = function(e) stop(e)
-      )
-      jsonfiles <- normalizePath(files)
-    }
+    jsonfiles <- normalizePath(file.path(files), mustWork = TRUE)
   }
 
-  if (parallel) {
-    future::plan(future::multisession)
+  # Old parallel argument
+  if (lifecycle::is_present(parallel)) {
+    lifecycle::deprecate_warn(
+      when = "1.1.1",
+      what = "fix_jsons(parallel)",
+      details = c(
+        i = "Use future::plan(\"multisession\") instead"
+      )
+    )
   }
 
   if (length(jsonfiles > 0)) {
     # Test if files are still corrupted
-    jsonfiles <- suppressWarnings(test_jsons(files = jsonfiles))
+    jsonfiles <- suppressWarnings(test_jsons(path = NULL, files = jsonfiles))
     n_fixed <- 0L
 
     if (jsonfiles[1] != "") {
       n_fixed <- fix_jsons_impl(jsonfiles)
     }
   } else {
-    return(message("No JSON files found."))
+    abort("No JSON files found.")
   }
 
-  if (parallel) {
-    future::plan(future::sequential)
-  }
-
-  return(message("Fixed ", sum(n_fixed), " files"))
+  inform(paste0("Fixed ", sum(n_fixed), " files"))
+  return(invisible(sum(n_fixed)))
 }
 
 fix_jsons_impl <- function(jsonfiles) {
   if (requireNamespace("progressr", quietly = TRUE)) {
-    p <- progressr::progressor(steps = length(jsonfiles))
+    p <- progressr::progressor(steps = length(jsonfiles)) # nolint
   }
 
   furrr::future_map_int(jsonfiles, ~ {
@@ -153,7 +157,7 @@ fix_jsons_impl <- function(jsonfiles) {
       p()
     }
 
-    lines <- suppressWarnings(vroom::vroom_lines(.x, altrep = FALSE, skip_empty_rows = TRUE))
+    lines <- readLines(.x, warn = FALSE, skipNul = TRUE)
     res <- 0L
 
     # Are there any illegal characters in the file? If so, these prevent readLines from reading
@@ -165,11 +169,12 @@ fix_jsons_impl <- function(jsonfiles) {
     }
 
     if (length(lines) == 0) {
-      return(0L)
+      return(res)
     } else if (length(lines) > 2) {
       eof <- lines[(length(lines) - 2):length(lines)]
     } else {
-      eof <- lines
+      eof <- character(3)
+      eof[seq_along(lines)] <- lines
     }
 
     res <- res + fix_eof(.x, eof, lines)
@@ -182,9 +187,6 @@ fix_jsons_impl <- function(jsonfiles) {
 }
 
 fix_illegal_ascii <- function(file, lines) {
-  # Read in the file using vroom, since this doesn't stop early when encountering illegal ASCII's
-  # lines <- suppressWarnings(vroom::vroom_lines(path, altrep = FALSE, skip_empty_rows = TRUE))
-
   # Find which lines contain non ASCII characters
   corrupt <- which(grepl("[^ -~]", lines))
 
@@ -198,29 +200,29 @@ fix_illegal_ascii <- function(file, lines) {
 }
 
 fix_eof <- function(file, eof, lines) {
-  last <- eof[length(eof)]
+  last <- eof[eof != ""]
+  last <- last[length(last)]
 
   # Cases where it can go wrong
-  if (length(eof) == 1 & eof[1] == "[") {
-    # 1: Similar to 8?
+  if (eof[1] == "[" && eof[2] == "" && eof[3] == "") {
+    # 1: If the last (and also only) line in the file is [ then it means the file was only
+    # opened but nothing was written. So, just close it with ] to have an empty JSON file.
     write("]", file, append = TRUE)
-  } else if (eof[2] == "," & eof[3] == "{}]") {
-    return(0L)
-  } else if (eof[1] == "{}]" & eof[2] == "]" & eof[3] == "]") {
+  } else if (eof[1] == "{}]" && eof[2] == "]" && eof[3] == "]") {
     # 2: Closing bracket applied thrice. Probably the result of a bad fix applied by this function
     write(lines[1:(length(lines) - 2)], file, append = FALSE)
-  } else if (eof[2] == "]" & eof[3] == "]") {
+  } else if (eof[2] == "]" && eof[3] == "]") {
     # 3: Closing bracket applied twice. Probably the result of a bad fix applied by this function
     write(lines[1:(length(lines) - 1)], file, append = FALSE)
   } else if (all(eof == "{}]")) {
     # 4: An empty object followed by a closing bracket is generally the result of a bad fix
     # applied by this function. This autocorrects it.
     write(lines[1:(length(lines) - 2)], file, append = FALSE)
-  } else if (eof[2] == "{}]" & eof[3] == "{}]") {
+  } else if (eof[2] == "{}]" && eof[3] == "{}]") {
     # 5: An empty object followed by a closing bracket is generally the result of a bad fix
     # applied by this function. This autocorrects it.
     write(lines[1:(length(lines) - 1)], file, append = FALSE)
-  } else if (eof[2] == "," & eof[3] == "]") {
+  } else if (eof[2] == "," && eof[3] == "]") {
     # 6: If the file closed with a comma, another object is expected
     # To fix this, rewrite the entire file without the comma as deleting characters
     # is not possible
@@ -230,13 +232,8 @@ fix_eof <- function(file, eof, lines) {
     # 7: Similar to 6, but without a closing ] for the file
     # Instead of rewriting the file, just add an empty object
     write("{}]", file, append = TRUE)
-  } else if (last == "[") {
-    # 8: If the last (and also only) line in the file is [ then it means the file was only
-    # opened but nothing was written. So, just close it with ] to have an empty JSON file.
-    write("]", file, append = TRUE)
-  } else if (nchar(last) > 3 &&
-    substr(last, nchar(last) - 1, nchar(last)) == "}}") {
-    # 9: Is the last line long (>3) and are the last two characters "}}"? Then somehow all
+  } else if (nchar(last) > 3 && substr(last, nchar(last) - 1, nchar(last)) == "}}") {
+    # 8: Is the last line long (>3) and are the last two characters "}}"? Then somehow all
     # we are missing is a closing bracket.
     write("]", file, append = TRUE)
   } else {
@@ -251,74 +248,78 @@ fix_eof <- function(file, eof, lines) {
 
 #' Test JSON files for being in the correct format.
 #'
-#' @description
-#' `r lifecycle::badge("stable")`
+#' @description `r lifecycle::badge("stable")`
+#'
+#' @inheritSection import Parallel
 #'
 #' @inheritSection import Progress
 #'
 #' @param path The path name of the JSON files.
 #' @param files Alternatively, a character list of the input files.
-#' @param db A mpathsenser database connection (optional). If provided, will be used to check
-#' which files are already in the database and check only those JSON files which are not.
+#' @param db A mpathsenser database connection (optional). If provided, will be used to check which
+#'   files are already in the database and check only those JSON files which are not.
 #' @param recursive Should the listing recurse into directories?
 #' @param parallel A logical value whether you want to check in parallel. Useful when there are a
-#' lot of files. If you have already used \code{\link[future]{plan}}, you can leave this parameter
-#' to \code{FALSE}.
+#'   lot of files. If you have already used \code{\link[future]{plan}}, you can leave this parameter
+#'   to \code{FALSE}.
 #'
-#' @return A message indicating whether there were any issues and a character vector of the file
-#' names that need to be fixed. If there were no issues, no result is returned.
+#'   `r lifecycle::badge("deprecated")` As functions should not modify the user's workspace,
+#'   directly toggling parallel support has been deprecated. Please use
+#'   \code{\link[future]{plan}("multisession")} before calling this function to use multiple
+#'   workers.
+#'
+#' @return A message indicating whether there were any issues and a character vector of
+#'   the file names that need to be fixed. If there were no issues, an invisible empty string is
+#'   returned.
 #' @export
 test_jsons <- function(path = getwd(),
                        files = NULL,
                        db = NULL,
                        recursive = TRUE,
-                       parallel = FALSE) {
-  if (!is.null(path) && !is.character(path)) {
-    stop("path must be a character string of the path name")
-  }
-  if (!is.null(files) && !is.character(files)) {
-    stop("files must be NULL or a character vector of file names")
+                       parallel = deprecated()) {
+  check_arg(path, "character", n = 1, allow_null = TRUE)
+  check_arg(files, "character", allow_null = TRUE)
+  check_arg(recursive, "logical", n = 1)
+
+  if (is.null(path) && is.null(files)) {
+    abort("`path` and `files` cannot be NULL at the same time.")
   }
 
-  if (is.null(files)) {
-    jsonfiles <- dir(
+  # Find all JSON files that are _not_ zipped Thus, make sure you didn't unzip them yet,
+  # otherwise this may take a long time
+  if (!is.null(path) && is.null(files)) {
+    jsonfiles <- list.files(
       path = path,
       pattern = "*.json$",
       all.files = TRUE,
       recursive = recursive,
       full.names = TRUE
     )
+  } else if (!is.null(path) && !is.null(files)) {
+    jsonfiles <- normalizePath(file.path(path, files), mustWork = TRUE)
   } else {
-    if (!missing(path)) {
-      tryCatch(
-        {
-          normalizePath(file.path(path, files), mustWork = TRUE)
-        },
-        error = function(e) stop(e)
-      )
-      jsonfiles <- normalizePath(file.path(files))
-    } else {
-      tryCatch(
-        {
-          normalizePath(files, mustWork = TRUE)
-        },
-        error = function(e) stop(e)
-      )
-      jsonfiles <- normalizePath(files)
-    }
+    jsonfiles <- normalizePath(file.path(files), mustWork = TRUE)
   }
 
+
   if (!is.null(db)) {
     processed_files <- get_processed_files(db)
     jsonfiles <- jsonfiles[!(jsonfiles %in% processed_files$file_name)]
   }
 
-  if (parallel) {
-    future::plan(future::multisession)
+  # Old parallel argument
+  if (lifecycle::is_present(parallel)) {
+    lifecycle::deprecate_warn(
+      when = "1.1.1",
+      what = "test_jsons(parallel)",
+      details = c(
+        i = "Use future::plan(\"multisession\") instead."
+      )
+    )
   }
 
   if (requireNamespace("progressr", quietly = TRUE)) {
-    p <- progressr::progressor(steps = length(jsonfiles))
+    p <- progressr::progressor(steps = length(jsonfiles)) # nolint
   }
 
   missing <- furrr::future_map_lgl(jsonfiles, ~ {
@@ -333,16 +334,12 @@ test_jsons <- function(path = getwd(),
     jsonlite::validate(str)
   }, .options = furrr::furrr_options(seed = TRUE))
 
-  if (parallel) {
-    future::plan(future::sequential)
-  }
-
   jsonfiles <- jsonfiles[!missing]
   if (length(jsonfiles) == 0) {
-    message("No issues found.")
+    inform("No issues found.")
     return(invisible(""))
   } else {
-    warning("There were issues in some files")
+    warn("There were issues in some files")
     return(normalizePath(jsonfiles))
   }
 }
@@ -355,6 +352,8 @@ test_jsons <- function(path = getwd(),
 #' Similar to \link[utils]{unzip}, but makes it easier to unzip all files in a given path
 #' with one function call.
 #'
+#' @inheritSection import Parallel
+#'
 #' @inheritSection import Progress
 #'
 #' @param path The path to the directory containing the zip files.
@@ -366,28 +365,36 @@ test_jsons <- function(path = getwd(),
 #' lot of files. If you have already used \code{future::plan}, you can leave this parameter to
 #' \code{FALSE}.
 #'
+#'   `r lifecycle::badge("deprecated")` As functions should not modify the user's workspace,
+#'   directly toggling parallel support has been deprecated. Please use
+#'   \code{\link[future]{plan}("multisession")} before calling this function to use multiple
+#'   workers.
+#'
 #' @return A message indicating how many files were unzipped.
 #' @export
 unzip_data <- function(path = getwd(),
                        to = NULL,
                        overwrite = FALSE,
                        recursive = TRUE,
-                       parallel = FALSE) {
-  if (is.null(path) || !is.character(path)) {
-    stop("path must be a character string")
-  }
-  if (is.null(overwrite) || !is.logical(overwrite)) {
-    stop("overwrite must be TRUE or FALSE")
-  }
-  if (is.null(recursive) || !is.logical(recursive)) {
-    stop("recursive must be TRUE or FALSE")
-  }
+                       parallel = deprecated()) {
+  check_arg(path, "character", n = 1)
+  check_arg(to, "character", allow_null = TRUE, n = 1)
+  check_arg(overwrite, "logical", n = 1)
+  check_arg(recursive, "logical", n = 1)
+
   if (is.null(to)) {
     to <- path
   }
 
-  if (parallel) {
-    future::plan(future::multisession)
+  # Old parallel argument
+  if (lifecycle::is_present(parallel)) {
+    lifecycle::deprecate_warn(
+      when = "1.1.1",
+      what = "unzip_data(parallel)",
+      details = c(
+        i = "Use future::plan(\"multisession\") instead"
+      )
+    )
   }
 
   unzipped_files <- 0
@@ -396,7 +403,7 @@ unzip_data <- function(path = getwd(),
     dirs <- list.dirs(path = path, recursive = TRUE)
 
     if (requireNamespace("progressr", quietly = TRUE)) {
-      p <- progressr::progressor(steps = length(dirs))
+      p <- progressr::progressor(steps = length(dirs)) # nolint
     }
 
     unzipped_files <- furrr::future_map_int(dirs, ~ {
@@ -410,14 +417,10 @@ unzip_data <- function(path = getwd(),
     unzipped_files <- unzip_impl(path, to, overwrite)
   }
 
-  if (parallel) {
-    future::plan(future::sequential)
-  }
-
   if (unzipped_files > 0) {
-    message(paste("Unzipped", unzipped_files, "files."))
+    inform(paste("Unzipped", unzipped_files, "files."))
   } else {
-    message("No files found to unzip.")
+    inform("No files found to unzip.")
   }
 }
 
@@ -446,7 +449,7 @@ unzip_impl <- function(path, to, overwrite) {
             exdir = to
           ))
         },
-        error = function(e) warning(paste0("Failed to unzip", x), call. = FALSE)
+        error = function(e) warn(paste0("Failed to unzip", x))
       )
     })
   }
