@@ -6,28 +6,41 @@ test_that("sensors-vec", {
 
 test_that("create_db", {
   filename <- tempfile("create", fileext = ".db")
-
   db <- create_db(path = NULL, filename)
-  DBI::dbDisconnect(db)
+  dbDisconnect(db)
   expect_true(file.exists(filename))
 
+  # Test merging path and filename
+  temp_file <- basename(tempfile())
+  expect_error(
+    {
+      db <- create_db(path = tempdir(), db_name = temp_file)
+      dbDisconnect(db)
+    },
+    NA
+  )
+
   # Test overwrite argument
   expect_error(
     {
       db <- create_db(path = NULL, filename, overwrite = TRUE)
-      DBI::dbDisconnect(db)
+      dbDisconnect(db)
     },
     NA
   )
+
   expect_error(
     {
       db <- create_db(path = NULL, filename, overwrite = FALSE)
-      DBI::dbDisconnect(db)
+      dbDisconnect(db)
     },
-    "Database .+?(?=\\.db)\\.db already exists\\. Use overwrite = TRUE to overwrite\\.",
+    "Database .+?(?=\\.db)\\.db already exists\\.",
     perl = TRUE
   )
 
+  # Test non-existing path
+  expect_error(create_db("foo", "bar"), "Directory .*?(?=foo)foo does not exist\\.", perl = TRUE)
+
   file.remove(filename)
 })
 
@@ -36,16 +49,16 @@ test_that("open_db", {
   expect_error(open_db(NULL, fake_db), "There is no such file")
 
   # Create a new (non-mpathsenser db)
-  db <- DBI::dbConnect(RSQLite::SQLite(), fake_db)
-  DBI::dbExecute(db, "CREATE TABLE foo(bar INTEGER, PRIMARY KEY(bar));")
-  DBI::dbDisconnect(db)
+  db <- dbConnect(RSQLite::SQLite(), fake_db)
+  dbExecute(db, "CREATE TABLE foo(bar INTEGER, PRIMARY KEY(bar));")
+  dbDisconnect(db)
   expect_error(open_db(NULL, fake_db), "Sorry, this does not appear to be a mpathsenser database.")
   file.remove(fake_db)
 
   path <- system.file("testdata", package = "mpathsenser")
   db <- open_db(path, "test.db")
-  expect_true(DBI::dbIsValid(db))
-  DBI::dbDisconnect(db)
+  expect_true(dbIsValid(db))
+  dbDisconnect(db)
 })
 
 test_that("copy_db", {
@@ -57,43 +70,53 @@ test_that("copy_db", {
     overwrite = TRUE,
     copy.mode = FALSE
   )
-  db <- open_db(NULL, test_db_name)
 
+  db <- open_db(NULL, test_db_name)
   new_db <- create_db(NULL, filename)
+
+  # Arguments
+  expect_warning(
+    copy_db(db, new_db, path = "foo"),
+    "The `path` argument of `copy_db\\(\\)` is deprecated as of mpathsenser 1.1.1."
+  )
+
+  expect_error(
+    copy_db(db, new_db, db_name = "foo"),
+    paste(
+      "The `db_name` argument of `copy_db\\(\\)` was deprecated in mpathsenser 1.1.1",
+      "and is now defunct."
+    )
+  )
+
+  # Invalid sensor
+  expect_error(
+    copy_db(db, new_db, sensor = "foo"),
+    "Sensor\\(s\\) foo not found."
+  )
+
   copy_db(db, new_db, sensor = "All")
   expect_equal(get_nrows(db), get_nrows(new_db))
   close_db(new_db)
   file.remove(filename)
 
   # Create new db and copy to it
-  copy_db(db, sensor = "Accelerometer", path = tempdir(), db_name = "copy.db")
-  new_db <- open_db(tempdir(), "copy.db")
+  new_db <- create_db(NULL, filename)
+  copy_db(db, new_db, sensor = "Accelerometer")
   true <- c(6L, rep(0L, 24))
   names(true) <- sensors
   expect_equal(get_nrows(new_db), true)
 
-  expect_error(
-    copy_db(db, sensor = "Accelerometer", path = tempdir(), db_name = "copy.db"),
-    paste0(
-      "A file in .+ with the name copy\\.db already exists\\. Please choose ",
-      "a different name or path or remove the file\\."
-    )
-  )
-  close_db(new_db)
-  file.remove(file.path(tempdir(), "copy.db"))
+  dbDisconnect(db)
+  dbDisconnect(new_db)
 
-  DBI::dbDisconnect(db)
-  expect_error(
-    copy_db(db, sensor = "Accelerometer", path = tempdir(), db_name = "copy.db"),
-    "Database connection is not valid"
-  )
   file.remove(test_db_name)
+  file.remove(filename)
 })
 
 test_that("close_db", {
   db <- open_db(system.file("testdata", package = "mpathsenser"), "test.db")
   expect_error(close_db(db), NA)
-  expect_false(DBI::dbIsValid(db))
+  expect_false(dbIsValid(db))
   expect_error(close_db(db), NA) # Invalid db
   rm(db)
   expect_error(close_db(db), NA) # db does not exist
@@ -110,7 +133,23 @@ test_that("index_db", {
   expect_error(index_db(db), "index idx_accelerometer already exists")
 
   # Cleanup
-  DBI::dbDisconnect(db)
+  dbDisconnect(db)
+  file.remove(filename)
+
+  expect_error(
+    index_db(db),
+    "Database connection `db` is not valid."
+  )
+})
+
+test_that("vacuum_db", {
+  # Create db
+  filename <- tempfile("foo", fileext = ".db")
+  db <- create_db(NULL, filename)
+  expect_error(vacuum_db(db), NA)
+
+  # Cleanup
+  dbDisconnect(db)
   file.remove(filename)
 })
 
@@ -120,15 +159,15 @@ test_that("add_study", {
   db <- create_db(NULL, filename)
 
   data <- data.frame(study_id = "12345", data_format = "mpathsenser")
-  expect_equal(add_study(db, data), 1)
+  expect_equal(add_study(db, data$study_id, data$data_format), 1)
 
   studies <- DBI::dbGetQuery(db, "SELECT * FROM Study")
   expect_equal(studies, data)
-  expect_equal(add_study(db, data), 0)
-  expect_equal(add_study(db, data.frame(foo = 1, bar = 2)), 0)
+  expect_equal(add_study(db, data$study_id, data$data_format), 0)
+  expect_equal(add_study(db, NULL, NULL), 0)
 
   # Cleanup
-  DBI::dbDisconnect(db)
+  dbDisconnect(db)
   file.remove(filename)
 })
 
@@ -138,15 +177,15 @@ test_that("add_participant", {
   db <- create_db(NULL, filename)
 
   data <- data.frame(participant_id = "12345", study_id = "12345")
-  DBI::dbExecute(db, "INSERT INTO Study VALUES('12345', 'mpathsenser')")
-  expect_equal(add_participant(db, data), 1)
+  dbExecute(db, "INSERT INTO Study VALUES('12345', 'mpathsenser')")
+  expect_equal(add_participant(db, data$participant_id, data$study_id), 1)
   participants <- DBI::dbGetQuery(db, "SELECT * FROM Participant")
   expect_equal(participants, data)
-  expect_equal(add_participant(db, data), 0)
-  expect_equal(add_participant(db, data.frame(foo = 1, bar = 2)), 0)
+  expect_equal(add_participant(db, data$participant_id, data$study_id), 0)
+  expect_equal(add_participant(db, NULL, NULL), 0)
 
   # Cleanup
-  DBI::dbDisconnect(db)
+  dbDisconnect(db)
   file.remove(filename)
 })
 
@@ -156,16 +195,16 @@ test_that("add_processed_file", {
   db <- create_db(NULL, filename)
 
   data <- data.frame(file_name = "12345.json", participant_id = "12345", study_id = "12345")
-  DBI::dbExecute(db, "INSERT INTO Study VALUES('12345', 'mpathsenser')")
-  DBI::dbExecute(db, "INSERT INTO Participant VALUES('12345', '12345')")
-  expect_equal(add_processed_files(db, data), 1)
+  dbExecute(db, "INSERT INTO Study VALUES('12345', 'mpathsenser')")
+  dbExecute(db, "INSERT INTO Participant VALUES('12345', '12345')")
+  expect_equal(add_processed_files(db, data$file_name, data$study_id, data$participant_id), 1)
   files <- DBI::dbGetQuery(db, "SELECT * FROM ProcessedFiles")
   expect_equal(files, data)
-  expect_equal(add_processed_files(db, data), 0)
-  expect_equal(add_processed_files(db, data.frame(foo = 1, bar = 2)), 0)
+  expect_equal(add_processed_files(db, data$file_name, data$study_id, data$participant_id), 0)
+  expect_equal(add_processed_files(db, NULL, NULL, NULL), 0)
 
   # Cleanup
-  DBI::dbDisconnect(db)
+  dbDisconnect(db)
   file.remove(filename)
 })
 
@@ -176,8 +215,7 @@ test_that("clear_sensors_db", {
   filename <- tempfile("foo", fileext = ".db")
   db <- create_db(NULL, filename)
 
-  db <- import(path, db = db, recursive = FALSE)
-  db <- open_db(NULL, filename)
+  suppressMessages(import(path, db = db, recursive = FALSE))
   original <- get_nrows(db)
   res <- clear_sensors_db(db)
   expect_type(res, "list")
@@ -186,7 +224,7 @@ test_that("clear_sensors_db", {
   expect_equal(sum(get_nrows(db)), 0L)
 
   # Cleanup
-  DBI::dbDisconnect(db)
+  dbDisconnect(db)
   file.remove(filename)
 })
 
@@ -199,7 +237,7 @@ test_that("get_processed_files", {
     study_id = c("-1", "test-study")
   )
   expect_equal(res, true)
-  DBI::dbDisconnect(db)
+  dbDisconnect(db)
 })
 
 test_that("get_participants", {
@@ -212,7 +250,7 @@ test_that("get_participants", {
   )
   expect_equal(res, true)
   expect_s3_class(res_lazy, "tbl_SQLiteConnection")
-  DBI::dbDisconnect(db)
+  dbDisconnect(db)
 })
 
 test_that("get_study", {
@@ -225,11 +263,11 @@ test_that("get_study", {
   )
   expect_equal(res, true)
   expect_s3_class(res_lazy, "tbl_SQLiteConnection")
-  DBI::dbDisconnect(db)
+  dbDisconnect(db)
 })
 
 test_that("get_nrows", {
   db <- open_db(system.file("testdata", package = "mpathsenser"), "test.db")
   expect_vector(get_nrows(db), integer(), length(sensors))
-  DBI::dbDisconnect(db)
+  dbDisconnect(db)
 })
